%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% file: plotThresholding.m
% author: Jakob Möderl (jakob.moederl@tugraz.at)
% institution: Graz University of Technology
% date: 29.11.2023
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Generate Thresholding Plots
%
% This file generates the plots of Figure 1 in the paper.
% The prior \hat{\gamma}_i and weights \bm{x}_i are calculated for a simple
% setup using different hyperprior PDFs.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


clear all;
close all;

addpath("fastBSBL/");

% generate signal and dictionary matrix
% fixed setting for lambda
lambda = 1;

% settings for gamma (and scaled jeffrey's prior)
prior_gamma(1).c = 0.1;
prior_gamma(1).a = 0;

prior_gamma(2).c = 0.5;
prior_gamma(2).a = 0;

prior_gamma(3).c = 1;
prior_gamma(3).a = 0;

prior_gamma(4).c = -0.1;
prior_gamma(4).a = 0;

prior_gamma(5).c = 1;
prior_gamma(5).a = 1;

prior_gamma(6).c = 0;
prior_gamma(6).a = 1;

% settings for jeffery's prior
prior_jeffrey(1).th = 1;
prior_jeffrey(2).th = 0.8;
prior_jeffrey(3).th = 0.67;
prior_jeffrey(4).th = 0.5;

% prio
prior_invGamma(1).b = 1;

% use real-valued signal model
rho=1/2;


% generate plot names
legend_name= {};
for idx_gamma = 1:numel(prior_gamma)
  legend_name = [legend_name, {sprintf('Gamma a=%1.1f, c=%1.1f', prior_gamma(idx_gamma).a, prior_gamma(idx_gamma).c)}];
end
for idx_jeffrey = 1:numel(prior_jeffrey)
  legend_name = [legend_name, {sprintf('Jeffrey th=%1.2f', prior_jeffrey(idx_jeffrey).th)}];
end
for idx_invGamma = 1:numel(prior_invGamma)
  legend_name = [legend_name, {sprintf('InvGamma b=%1.1f', prior_invGamma(idx_invGamma).b)}];
end




% iterate over different block sizes
BlockSize_array = [1, 2, 5, 10];
for idx_blockSize = 1:numel(BlockSize_array)
  BlockSize = BlockSize_array(idx_blockSize);

  % normalizing function for plots
  x_plot_fun = @(x) norm(x)/sqrt(BlockSize);

  % generate dictionary matrix
  Phi = eye(BlockSize);
  
  % different scalings for plot
  alpha = [0:0.002:5].';
  y_plot = zeros(numel(alpha), 1);
  x_plot_invGamma = zeros(numel(alpha), numel(prior_invGamma));
  x_plot_Jeffrey = zeros(numel(alpha), numel(prior_jeffrey));
  x_plot_Gamma = zeros(numel(alpha), numel(prior_gamma));

  % iterate over scalings
  for idx_phi_y = 1:numel(alpha)
    % measurement vector 
    y = ones(BlockSize, 1)*alpha(idx_phi_y);
    % vector for the x-axis of the plot
    y_plot(idx_phi_y) = x_plot_fun(y);
    
    % calculate polynomial coefficients
    Sigma_bar = inv(lambda*Phi'*Phi);
    [U, S] = eig(Sigma_bar);
    q = lambda * U'*Sigma_bar*y;
    s = diag(S);
    [A, B] = getPolynomialsAB(abs(q).^2, s);
    
    % calculate prior precisions \hat{\gamma}_i for all priors
    %% Gamma
    for idx_Gamma = 1:numel(prior_gamma)
      prior_c = prior_gamma(idx_Gamma).c;
      prior_a = prior_gamma(idx_Gamma).a;
      alpha_i_ast = calcGamma_Gamma(A, B, prior_a, prior_c, rho, 1, 0);
      % calculate weights
      if isinf(alpha_i_ast)
        x = zeros(BlockSize, 1);
      else
        Sigma = inv(lambda*(Phi'*Phi) + min(alpha_i_ast)*eye(BlockSize));
        x = lambda * Sigma*y;
      end
      % vector to plot later on
      x_plot_Gamma(idx_phi_y, idx_Gamma) = x_plot_fun(x);
    end


    %% Jeffrey's with threshold -1dB = 0.8
    for idx_jeffrey = 1:numel(prior_jeffrey)
      th = prior_jeffrey(idx_jeffrey).th;
      alpha_i_ast = calcGamma_Jeffrey(A, B, th, 0);
      % calculate weights
      if isinf(alpha_i_ast)
        x = zeros(BlockSize, 1);
      else
        Sigma = inv(lambda*(Phi'*Phi) + min(alpha_i_ast)*eye(BlockSize));
        x = lambda * Sigma*y;
      end
      % vector to plot later on
      x_plot_Jeffrey(idx_phi_y, idx_jeffrey) = x_plot_fun(x);
    end

    %% Inverse Gamma
    for idx_InvGamma = 1:numel(prior_invGamma)
      prior_b = prior_invGamma(idx_InvGamma).b;
      alpha_i_ast = calcGamma_InverseGamma(A, B, prior_b, rho, 1, 0);
      % calculate weights
      if isinf(alpha_i_ast)
        x = zeros(BlockSize, 1);
      else
        Sigma = inv(lambda*(Phi'*Phi) + min(alpha_i_ast)*eye(BlockSize));
        x = lambda * Sigma*y;
      end
      % vector to plot later on
      x_plot_invGamma(idx_phi_y, idx_InvGamma) = x_plot_fun(x);
    end
  end


  %% plotting
  fig(idx_blockSize)=figure(idx_blockSize);
  hold on;
  plot(y_plot, x_plot_Gamma, ':');
  plot(y_plot, x_plot_Jeffrey, '--');
  plot(y_plot, x_plot_invGamma);
  plot(y_plot, y_plot, '--', 'Color', 0.5*ones(3, 1))
  legend(legend_name{:}, 'x=y', 'Location', 'North West')
  title(sprintf('d_i=%d', BlockSize));
  xlim([0, 5])
  ylim([0, 5])

  

end


%% Matlab 2 tikz
EXPORT_TIKZ = false;
if EXPORT_TIKZ
  WORKSTATION = 2;
  for idx_blockSize = 1:numel(BlockSize_array)
    BlockSize = BlockSize_array(idx_blockSize);

    if WORKSTATION==1
      addpath(genpath('C:\Users\Jakob\Documents\MATLAB\matlab2tikz'))
    elseif WORKSTATION==2
      addpath(genpath('/afs/spsc.tugraz.at/user/jakob/MATLAB Add-Ons/matlab2tikz/'))
    end


    filename_tex = sprintf('pgf/threhsolding-K%d.tex', BlockSize);
    fig_plot = fig(idx_blockSize);

    figure(fig_plot);
    cleanfigure();

    % tikz picture
    mat2tikz.standalone = false;
    mat2tikz.externaldata = true;
    mat2tikz.showinfo = true;
    mat2tikz.showhidden = false;

    mat2tikz.width  = '\figurewidth';
    mat2tikz.height = '\figureheight';
    mat2tikz.datapath = '\datapath';

    mat2tikz.extraaxisoptions = {'%extraAxisOptions'};
    mat2tikz.extraaxisoptions = [mat2tikz.extraaxisoptions, ...
      {'% unit vector ratio=1 1 1'}];

    % extra code to align minus and plus in ticks, set all text sizes
    mat2tikz.extracode = {
      ' ', ...
      '\pgfplotsset{every axis/.append style={', ...
      '  label style={font=\scriptsize},', ...
      '  legend style={font=\scriptsize},', ...
      '  tick label style={font=\scriptsize},', ...
      '  xticklabel={', ...
      '    % test if the x value is below zero', ...
      '    \ifdim \tick pt < 0pt', ...
      '      % if yes, calculate the absolute value', ...
      '      \pgfmathparse{abs(\tick)}%', ...
      '      % and print first a minus sign in a zero-width box, followed by the absolute value', ...
      '      \llap{$-{}$}\pgfmathprintnumber{\pgfmathresult}', ...
      '   \else', ...
      '     % if no, print as usual', ...
      '      \pgfmathprintnumber{\tick}', ...
      '   \fi', ...
      '}}}', ...
      ' '};

    cleanfigure();
    matlab2tikz(filename_tex, ...
      'figurehandle',fig_plot, ...
      'standalone',mat2tikz.standalone, ...
      'showinfo',mat2tikz.showinfo, ...
      'externaldata',mat2tikz.externaldata, ...
      'extraCode',mat2tikz.extracode, ...
      'extraAxisOptions',mat2tikz.extraaxisoptions, ...
      'relativeDataPath',mat2tikz.datapath, ...
      'Height',mat2tikz.height, ...
      'Width',mat2tikz.width, ... %set in latex as \definelengeth{} and later \setlength{}
      'showHiddenStrings',mat2tikz.showhidden)

  end
end