%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% file: testFastBSBL.m
% author: Jakob Möderl (jakob.moederl@tugraz.at)
% institution: Graz University of Technology
% date: 29.11.2023
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Fast BSBL Demo Example
%
% This file contains a demo example on how to use the fast variational
% block-sparse Bayesian learning algorithm.
% We use a 200x400 dictionary matrix with Gaussian columns multiplied with
% 40 blocks of weights of size d_i=10 each.
% From the 40 blocks, 8 blocks are chosen at random locations to be nonzero
% and the remaining 32 blocks of weights are zero.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all;
close all;

addpath("fastBSBL/");
addpath("BSBL_public/");

% generate signal and dictionary matrix
% number of samples in the measurements
Nsig = 200;
% Size of each block
BlockSize = 10;
% Number of blocks K
Nblock = floor(2*Nsig/BlockSize);
% Number of active blocks
Nactive = round(0.2*Nblock);
% K x 1 vector of block sizes
di = BlockSize*ones(Nblock, 1);
% start indices of the blocks in the weight vector
blk_start = cumsum([1; di(1:end-1)]);
% Number of weights
M = sum(di);

% SNR to calculate the noise variance
SNR_dB = 15;

% Coeffient \beta in case the weights are generated by a first-order
% autoregressive process. If beta is not 0, then calc_sigma must be set to
% "full" for the fastBSBL algorithm.
beta = 0.0;
% Weight covariances resulting from \beta
Source_covar = toeplitz(beta.^(0:BlockSize-1));
% Intra-block precision 
B = inv(Source_covar);
L_source = chol(Source_covar)';

% draw random indices for nozero weights and generate them according to the AR
% process
x_true = complex(zeros(M, 1));
for blk_idx = randperm(numel(di), Nactive)
  idx_x = blk_start(blk_idx):(blk_start(blk_idx)+di(blk_idx)-1);
  x_true(idx_x) = L_source * randn(di(blk_idx), 1);
end

% Generate dictionary matrix
Phi = randn(Nsig, M);
% Normalize all columns to have same power
for idx_column = 1:size(Phi, 2)
  Phi(:, idx_column) = Phi(:, idx_column)/sqrt(sum(abs(Phi(:, idx_column)).^2));
end

% Generate (noiseless) measurement
y = Phi * x_true;

% Signal Power
Ey = sum(abs(y).^2);
% Desired noise Power
En = Ey/db2pow(SNR_dB);
% Noise scaling to achieve desired SNR
n = sqrt(En/Nsig) * randn(Nsig, 1);

% Add noise to measurements
y = y+n;

% function handle for MSE calculation
calcNMSE = @(x) sum(abs(x-x_true).^2)/sum(abs(x_true).^2);

%% FastBSBL
fprintf('Fast BSBL: ')
tic;
[x_fastBSBL, Sigma, lambda, gamma, elbo] = fastBSBL(y, Phi, blk_start, 'prior_pdf_type', 'Gamma', ...
  'prior_c', 1, 'prior_b', 0, 'threshold', 1, ...
  'calc_sigma' ,'full', 'B', B, 'do_print', false);
toc;

fprintf('MSE fastBSBL: %f\n', calcNMSE(x_fastBSBL));



%% BSBL-BO
learn_lambda = 2;
fprintf('BSBL-BO: ')
tic
res_BSBL_BO = BSBL_EM(Phi, y, blk_start, learn_lambda, 'LEARNTYPE', 2, 'B', Source_covar, 'PRUNE_GAMMA', 0.001, 'PRINT', false);
x_bsblBO = res_BSBL_BO.x;
toc

fprintf('MSE BSBL-BO:  %f\n', calcNMSE(x_bsblBO));


%% Plotting
figure;
hold on;
stem(abs(x_true));
stem(abs(x_fastBSBL));
stem(abs(x_bsblBO));
% stem(abs(X_tf));
ylabel('|x|');
xlabel('Weights');
legend('true', 'fastBSBL', 'BSBL-BO');















